from django.views.generic.detail import DetailView
from django.views.generic.list import ListView

from .models import Post


class PostListView(ListView):
    model = Post
    template_name = 'djtools/blog/post_list.html'


class PostDetailView(DetailView):
    model = Post
    template_name = 'djtools/blog/post_detail.html'



from django.urls import path

from . import views

app_name = 'blog'
urlpatterns = [
    path('', views.PostListView.as_view(), name='list'),
    path('<slug:slug>/', views.PostDetailView.as_view(), name='detail'),
]



from django.contrib import admin

from .models import Post, PostPhoto


class PostPhotoInline(admin.TabularInline):
    model = PostPhoto
    extra = 1


@admin.register(Post)
class PostAdmin(admin.ModelAdmin):
    list_display = [
        'title',
        'created',
        'updated',
    ]

    list_filter = [
        'created',
    ]

    search_fields = [
        'title',
        'subtitle',
        'body',
    ]

    prepopulated_fields = {
        'slug': ['title'],
    }

    readonly_fields = [
        'created',
        'updated',
    ]

    inlines = [
        PostPhotoInline,
    ]



from django.db import models
from django.utils.text import slugify

from . import utils


class BaseModel(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Post(BaseModel):
    title = models.CharField(max_length=255)
    slug = models.SlugField(unique=True, max_length=255, db_index=True)
    subtitle = models.CharField(max_length=255, blank=True, null=True)
    body = models.TextField()
    image = models.ImageField(upload_to=utils.photo_upload_to)

    class Meta:
        ordering = ['-created']

    def __str__(self):
        return self.title

    def save(self, *args, **kwargs):
        self.slug = slugify(self.title)
        super().save(*args, **kwargs)

    @classmethod
    def get_last_post(cls):
        try:
            last_post = Post.objects.all()[0]
        except IndexError:
            last_post = None

        return last_post


class PostPhoto(BaseModel):
    post = models.ForeignKey(Post, on_delete=models.CASCADE)
    image = models.ImageField(upload_to=utils.post_photo_upload_to)
    title = models.CharField(
        max_length=255,
        help_text="This text will be used for the img alt attribute and for the image caption",
        blank=True,
        null=True,
    )
    order = models.PositiveIntegerField(default=0)
    featured = models.BooleanField(default=False)

    class Meta:
        ordering = (
            '-featured',
            'order',
            '-updated',
        )
        index_together = [
            ['featured', 'order', 'updated'],
        ]

    def __str__(self):
        if self.title:
            return self.title
        return super().__str__()



def upload_to(slug, file_name):
    return f"posts/{slug}/{file_name}"


def photo_upload_to(instance, file_name):
    return upload_to(instance.slug, file_name)


def post_photo_upload_to(instance, file_name):
    return upload_to(instance.post.slug, file_name)



# Generated by Django 2.1.5 on 2019-01-11 10:08

from django.db import migrations, models
import django.db.models.deletion
import djtools.blog.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255)),
                ('slug', models.SlugField(max_length=255, unique=True)),
                ('subtitle', models.CharField(blank=True, max_length=255, null=True)),
                ('body', models.TextField()),
                ('image', models.ImageField(upload_to=djtools.blog.utils.photo_upload_to)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
        migrations.CreateModel(
            name='PostPhoto',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('image', models.ImageField(upload_to=djtools.blog.utils.post_photo_upload_to)),
                ('title', models.CharField(blank=True, help_text='This text will be used for the img alt attribute and for the image caption', max_length=255, null=True)),
                ('order', models.PositiveIntegerField(default=0)),
                ('featured', models.BooleanField(default=False)),
                ('post', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djtools_blog.Post')),
            ],
            options={
                'ordering': ('-featured', 'order', '-updated'),
            },
        ),
        migrations.AlterIndexTogether(
            name='postphoto',
            index_together={('featured', 'order', 'updated')},
        ),
    ]
